@@ -83,7 +83,7 @@ extern BOOL gISearch;        // インクリメンタルサーチ中かどうか
 void isearch_init();         // インクリメンタルサーチ初期化
 void isearch_final();        // インクリメンタルサーチ解放
 
-void isearch_input(int meta, int key); // インクリメンタルサーチキー入力
+void isearch_input(int meta, int* keybuf, int keybuf_size); // インクリメンタルサーチキー入力
 void isearch_view();                  // インクリメンタルサーチ描写
 
 BOOL IsISearchExploreChar(int meta, int key);
@@ -194,6 +194,7 @@ void gui_init();
 
 void xstart_color();
 int xgetch(int* meta);
+void xgetch_utf8(int* meta, ALLOC int** key, int* key_size);
 void xinitscr();
 void xendwin();
 void xclear();
