@@ -359,7 +359,7 @@ BOOL filer_add_history(int dir, char* path)
 // result --> vector_obj of string_obj
 BOOL filer_get_hitory(sObject* result, int dir)
 {
-    ASSERT(TYPE(result) == T_VECTOR);
+    ASSERT(STYPE(result) == T_VECTOR);
 
     if(dir < 0 || dir >= vector_count(gDirs)) {
         return FALSE;
