@@ -78,6 +78,78 @@ int xgetch(int* meta)
     }
 }
 
+void xgetch_utf8(int* meta, ALLOC int** key, int* key_size)
+{
+    fd_set mask;
+    fd_set read_ok;
+
+    FD_ZERO(&mask);
+    FD_SET(0, &mask);
+
+    *key = MALLOC(sizeof(int)*128);
+    *key_size = 0;
+    int key_alloc_size = 128;
+
+    BOOL input_started = FALSE;
+
+    while(1) {
+        read_ok = mask;
+
+        struct timeval tv;
+        tv.tv_sec = 0;
+        tv.tv_usec = 0;
+
+        select(5, &read_ok, NULL, NULL, &tv);
+
+        if(FD_ISSET(0, &read_ok)) {
+            input_started = TRUE;
+
+            int k = getch();
+
+            if(k == 27) {
+                k = getch();
+
+                *meta = 1;
+                (*key)[0] = k;
+                (*key)[1] = 0;
+                *key_size = 1;
+
+                return;
+            }
+            else if(k >= KEY_MIN && k <= KEY_MAX) { // curses key
+                *meta = 0;
+                (*key)[0] = k;
+                (*key)[1] = 0;
+                *key_size = 1;
+
+                return;
+            }
+            else if(k >= 128) { // utf8 characters
+                if(*key_size >= key_alloc_size) {
+                    key_alloc_size *= 2;
+                    *key = REALLOC(*key, sizeof(int)*key_alloc_size);
+                }
+                (*key)[(*key_size)++] = k;
+                (*key)[*key_size] = 0;
+            }
+            else {
+                *meta = 0;
+                (*key)[0] = k;
+                (*key)[1] = 0;
+                *key_size = 1;
+
+                return;
+            }
+        }
+        else {
+            if(input_started) {
+                *meta = 0;
+                return;
+            }
+        }
+    }
+}
+
 ///////////////////////////////////////////////////
 // エラーメッセージ表示
 ///////////////////////////////////////////////////
