@@ -49,13 +49,6 @@ void ISearchClear()
 ///////////////////////////////////////////////////
 // 内部定義
 ///////////////////////////////////////////////////
-/// 入力された文字列がインクリメンタルサーチ向きかどうか ///
-static BOOL is_isearch_char(int key)
-{
-    return key == 9 || (key >= ' ' && key <= '~')
-                && key != '\\' || key >= 128;
-}
-
 static char* mystrcasestr(const char *haystack, const char *needle)
 {
     char* p;
@@ -194,8 +187,10 @@ void isearch_final()
 ///////////////////////////////////////////////////
 // インクリメンタルサーチキー入力
 ///////////////////////////////////////////////////
-void isearch_input(int meta, int key)
+void isearch_input(int meta, int* keybuf, int keybuf_size)
 {
+    int key = keybuf[0];
+
     if(key == 8 || key == KEY_BACKSPACE || key == KEY_DC) {
         string_erase(gInputFileName, string_length(gInputFileName)-1, 1);
     }
@@ -231,16 +226,40 @@ void isearch_input(int meta, int key)
             match_next(filer_dir(adir())->mCursor+1);
         }
     }
-    else if(!is_isearch_char(key)) {
-        gISearch = FALSE;
-        gISearchPartMatch = FALSE;
-        string_put(gInputFileName, "");
+#ifdef HAVE_LIB_CURSESW
+    else if(key >= 128 && !(key >= KEY_MIN && key <= KEY_MAX)) {
+        unsigned char* keybuf2 = MALLOC(sizeof(unsigned char)*keybuf_size + 1);
+        int i;
+        for(i=0; i<keybuf_size; i++) {
+            keybuf2[i] = (unsigned char)keybuf[i];
+        }
+        keybuf2[i] = 0;
+
+        string_push_back3(gInputFileName, keybuf2, keybuf_size);
+
+        FREE(keybuf2);
+
+        if(!gISearchPartMatch) {
+            if(!match_next(0)) {
+                gISearchPartMatch = TRUE;
+
+                if(!match_next(0)) {
+                    string_erase(gInputFileName, string_length(gInputFileName)-1, 1);
+
+                }
+
+                gISearchPartMatch = FALSE;
+            }
+        }
     }
-    else {
-        if(key == 9)
+#endif
+    else if(key == 9 || (key >= ' ' && key <= '~' && key != '\\')) {
+        if(key == 9) {
             string_push_back2(gInputFileName, ' ');
-        else
-            string_push_back2(gInputFileName, (char)key);
+        }
+        else {
+            string_push_back2(gInputFileName, (unsigned char)key);
+        }
 
         if(!gISearchPartMatch) {
             if(!match_next(0)) {
@@ -261,6 +280,11 @@ void isearch_input(int meta, int key)
             }
         }
     }
+    else {
+        gISearch = FALSE;
+        gISearchPartMatch = FALSE;
+        string_put(gInputFileName, "");
+    }
 }
 
 ///////////////////////////////////////////////////
